@@ -65,8 +65,6 @@
 #include "mkfs/common.h"
 #include "mkfs/rootdir.h"
 
-#include "libbtrfs/ctree.h"
-
 struct mkfs_allocation {
 	u64 data;
 	u64 metadata;
@@ -1668,7 +1666,8 @@ int BOX_MAIN(mkfs)(int argc, char **argv)
 	}
 
 	oca.filename = file;
-	oca.flags = OPEN_CTREE_WRITES | OPEN_CTREE_TEMPORARY_SUPER;
+	oca.flags = OPEN_CTREE_WRITES | OPEN_CTREE_TEMPORARY_SUPER |
+		    OPEN_CTREE_EXCLUSIVE;
 	fs_info = open_ctree_fs_info(&oca);
 	if (!fs_info) {
 		error("open ctree failed");
