@@ -7,7 +7,7 @@
 # - various configure options
 #
 # Arguments:
-# - (first arugment) --ccache - enable ccache for build which can speed up
+# - (first argument) --ccache - enable ccache for build which can speed up
 #    rebuilding same files if the options do not affect them, the ccache will
 #    be created in the toplevel git directory
 # - anything else will be passed to 'make', eg. define CC, D, V
@@ -26,7 +26,7 @@ check_result() {
 
 	ret=$1
 
-	str="RESULT of target($target) conf($conf): "
+	str="RESULT of target($target) conf($conf) CFLAGS($CFLAGS): "
 	case $ret in
 		0) str="$str OK";;
 		*) str="$str FAIL";;
@@ -39,13 +39,19 @@ $str"
 buildme() {
 	make clean-all
 
-	./autogen.sh && configure "$conf" || die "configure not working with: $@"
+	./autogen.sh && CFLAGS="$CFLAGS" configure "$conf" || die "configure not working with: $@"
 	$make clean
 	$make $opts $target
 	check_result "$?"
 	echo "VERDICT: $verdict"
 }
 
+buildme_cflags() {
+	CFLAGS="$1"
+	buildme
+	CFLAGS=''
+}
+
 build_make_targets() {
 	# defaults
 	target=
@@ -85,6 +91,8 @@ jobs=16
 opts="-j${jobs} $@"
 verdict=
 target=
+export CLFAGS
+CFLAGS=
 
 conf=
 build_make_targets
@@ -122,6 +130,19 @@ build_make_targets
 conf='--with-crypto=libkcapi'
 build_make_targets
 
+conf='--with-crypto=botan'
+build_make_targets
+
+conf='--with-crypto=openssl'
+build_make_targets
+
+# Old architectures
+conf='--with-crypto=builtin'
+buildme_cflags '-march=core2'
+
+conf='--with-crypto=builtin'
+buildme_cflags '-march=x86-64-v3'
+
 # debugging builds, just the default targets
 target='D=1'
 buildme
